(* kocka.pas *)

{ Tetszleges trbeli alakzat forgatsa trben:

  Np - a cscspontok szma
  Pontok[1..Np] a cscspontok tmbje
  Ne - az lek szma
  Elek[1..Ne] az lek tmbje

  Origo a forgats kzppontja,
  Az akakzat forgatst az EVector forgatsval vgezzk
}

program kocka;
uses crt, graph;

type  VilagPont=record
        x, y, z : real;
      end;

type  KepPont=record
        w, h : integer;
      end;

type Vonal=record
        p, q : integer;
     end;

var
  { Tr - a traszformcis mtrix }
  Tr : array [1..3,1..3] of real;
  tav : real;
  maxx, maxy : integer;
  meret : real;

const
{ A cscspontok szma }
  Np = 8;

{ Az lek szma }
  Ne = 12;

{ Az alakzat cscspontjai }
 Pontok : array[1..Np] of VilagPont =( (x:0; y:0; z:0),
		                       (x:1; y:0; z:0),
		                       (x:1; y:1; z:0),
		                       (x:0; y:1; z:0),
		                       (x:0; y:1; z:1),
		                       (x:1; y:1; z:1),
		                       (x:1; y:0; z:1),
		                       (x:0; y:0; z:1));

{ Az alakzat lei ( a cscspont-indexszekkel megadva) }
Elek : array [1..Ne] of Vonal = ((p:1; q:2),
                                 (p:2; q:3),
		                 (p:3; q:4),
		                 (p:4; q:1),
		                 (p:4; q:5),
		                 (p:5; q:6),
		                 (p:6; q:7),
		                 (p:7; q:8),
		                 (p:8; q:5),
		                 (p:3; q:6),
		                 (p:2; q:7),
		                 (p:1; q:8));
{ Mretoszt }
 MD = 16;

{ ------------------------------------}
{ A transzformcis mtrix feltltse }
{ ------------------------------------}
procedure Feltolt (  A, B, E : VilagPont; l1, l2, l3 : real);
begin
  Tr[1,1]:= l1 * (B.y * E.z - E.y * B.z);
  Tr[1,2]:= l1 * (E.x * B.z - B.x * E.z);
  Tr[1,3]:= l1 * (B.x * E.y - E.x * B.y);
  Tr[2,1]:= l2 * (E.y * A.z - A.y * E.z);
  Tr[2,2]:= l2 * (A.x * E.z - E.x * A.z);
  Tr[2,3]:= l2 * (E.x * A.y - A.x * E.y);
  Tr[3,1]:= l3 * (A.y * B.z - B.y * A.z);
  Tr[3,2]:= l3 * (B.x * A.z - A.x * B.z);
  Tr[3,3]:= l3 * (A.x * B.y - B.x * A.y);
end;

{ ---------------}
{ Transzformci }
{ ---------------}
procedure Transzformacio (O, E: VilagPont; la, lb: real);
var
  A, B : VilagPont;
  le : real;
begin
  meret := (la+lb)/2;

  E.x:= E.x - O.x;
  E.y:= E.y - O.y;
  E.z:= E.z - O.z;

  A.x:= -E.y;
  A.y:= +E.x;
  A.z:= 0;

  B.x:= E.y * A.z - A.y * E.z;
  B.y:= E.z * A.x - A.z * E.x;
  B.z:= E.x * A.y - A.x * E.y;

  tav:=sqrt (A.x * A.x + A.y * A.y + A.z * A.z );
  A.x:= A.x * la / tav;
  A.y:= A.y * la / tav;
  A.z:= A.z * la / tav;

  tav:= sqrt ( B.x * B.x + B.y * B.y + B.z * B.z );
  B.x:= B.x * lb / tav;
  B.y:= B.y * lb / tav;
  B.z:= B.z * lb / tav;

  tav:= + A.x * B.y * E.z + B.x * E.y * A.z + E.x * A.y * B.z
      - A.x * E.y * B.z - B.x * A.y * E.z - E.x * B.y * A.z ;

  le:=  E.x * E.x + E.y * E.y + E.z * E.z;

  Feltolt ( A, B, E, la*la, lb*lb, le );
end;

{ ---------------------}
{ Lekpzs a kpernyre }
{ ---------------------}
procedure Lekepzes(O, P : VilagPont; var Q : KepPont);
var
  V : VilagPont;
begin
  P.x:= P.x -O.x;
  P.y:= P.y -O.y;
  P.z:= P.z -O.z;
  V.x:= (Tr[1,1] * P.x + Tr[1,2] * P.y + Tr[1,3] * P.z) / tav;
  V.y:= (Tr[2,1] * P.x + Tr[2,2] * P.y + Tr[2,3] * P.z) / tav;
  V.z:= (Tr[3,1] * P.x + Tr[3,2] * P.y + Tr[3,3] * P.z) / tav;

  Q.w:= round(V.x / (1 - V.z / (maxx+maxy) / MD));
  Q.h:= round(V.y / (1 - V.z / (maxx+maxy) / MD));

  Q.w:= maxx div 2 + Q.w;
  Q.h:= maxy div 2 - Q.h;
end;

{ -----------}
{ Vonalhzs }
{ -----------}
procedure Line (P1, P2 : KepPont; color : integer);
begin
  setcolor (color);
  moveto (P1.w, P1.h);
  lineto (P2.w, P2.h);
end;


{ A fprogram }
var
  EVektor, Origo : VilagPont;
  Elozo, Kovetkezo : array[1..Np] of KepPont;
  KP : KepPont;
  u, v, d0 : real;
  gd, gm, i, db : integer;

begin
   detectgraph(gd,gm);
   if (gd<>EGA) and (gd<>VGA) then
      begin
       writeln(#9'Csak EGA, vagy VGA grafikval rendelkez gpen futtathat!');
       halt(1);
      end;

  initgraph(gd,gm,'c:\tp\bgi');
  if graphresult<>grok then
                          begin
                             writeln('Grafikus hiba');
                             halt(1);
                          end;
  maxx:= getmaxx;
  maxy:= getmaxy;

  Origo.x:= 0;  { 0.5 }
  Origo.y:= 0;  { 0.5 }
  Origo.z:= 0;  { 0.5 }

  for i:= 1 to  Np do
   begin
      Kovetkezo[i].w:= 0;
      Kovetkezo[i].h:= 0;
   end;

  d0:= (maxx + maxy) / MD;
  u:= 4.0;
  db:=0;

  repeat
    { Az EVector forgatsa }
    if db<200 then v:= 0
              else v:= 2 * u;
    if db=500 then
                  begin
                    Origo.x:= 0.5;
                    Origo.y:= 0.5;
                    Origo.z:= 0.5;
                  end;
    EVektor.x:= d0 * cos(u) * cos(v);
    EVektor.y:= d0 * sin(u) ;
    EVektor.z:= d0 * sin(v) ;

    if (EVektor.x<>Origo.x) or (EVektor.y<>Origo.y) then
    begin
      Transzformacio (Origo, EVektor, d0, d0);
      for i:= 1 to  Np do
        begin
	  Elozo[i] := Kovetkezo[i];
          Lekepzes(Origo, Pontok[i], KP);
          Kovetkezo[i]:= KP;
        end;

      { A rgi vonalak letrlse }
      for i:= 1 to Ne do
          Line ( Elozo[Elek[i].p], Elozo[Elek[i].q], 0 );

      { Az j vonalak letrlse }
      for i:= 1 to Ne do
	Line ( Kovetkezo[Elek[i].p], Kovetkezo[Elek[i].q], LightGreen);
    end;
    delay(20);
    u := u + 0.05;
    inc(db);
  until keypressed;
  readkey;
  closegraph;
end.
