(* MAKEPCX.PAS*)

unit makePCX;
{ --------------------- Interface ----------------- }
interface

 type Str80 = string [80];

 { A VGA kpb PCX-fjlt elllt eljrs }
 procedure WritePCX (Name:Str80; BackColor : byte);

{ ------------------- Implementation ------------ }
implementation

uses graph;

const
     RED1   = 0;
     GREEN1 = 1;
     BLUE1  = 2;

type
    ArrayPal   = array [0..15, RED1..BLUE1] of byte; { 16 szn paletta }

const
 INTENSTART =   $5;
 BLUESTART  =  $55;
 GREENSTART =  $A5;
 REDSTART   =  $F5;

type
 Pcx_Header = record
   Manufacturer: byte;    {  mindig 10 - PCX-fjl }
   Version: byte;         { 2 - rgi PCX - nincs paletta (nem hasznlt),
                            3 - nincs paletta,
                            4 - rgi Microsoft Windows - nincs paletta
                            5 - palettval }
   Encoding: byte;         { 1 PCX-kdols }
   Bits_per_pixel: byte;   { hny bit (skonknt) reprezentl egy pixelt:
                             1, 2, 4, vagy 8 }
   Xmin: integer;          { a kpablak mretei }
   Ymin: integer;
   Xmax: integer;
   Ymax: integer;

   Hdpi: integer;          { a kp felbontsa (dpi) }
   Vdpi: integer;          { psztzsi felbonts - 300 az alaptk }

   ColorMap: ArrayPal;     { az RGB-paletta (16 vagy kevesebb szn esetn)
                             a 256-szn paletta a fjl vgn helyezkedik el }

   Reserved: byte;         { 0 }

   Nplanes: byte;          { a bitskok szma }

   B_l_p: integer;         { Bytes/line/plane
                             A bitsk egy sornak hossza (bjt)
                             Pros szm kell legyen! }

   PaletteInfo: integer;   { 1 = feket-fehr vagy sznes kp,
                             2 = szrke kp }

   HscreenSize: integer;   { a PC Paintbrush IV Plus ver 1.0 hasznlja  }
   VscreenSize: integer;   { PC Paintbrush IV ver 1.02 hasznlja }

   Filler: array [74..127] of byte;     { 0-k}
 end;

 Array80    = array [1..80]        of byte;
 { a 6 extra bjtot a BGI teszi a kp el (mretinfo) }
 ArrayLnImg = array [1..326]       of byte;
 ArrayLnPCX = array [1..4]         of Array80;

var
 PCXName   : File;
 Header    : Pcx_Header; { a PCX-fjl fejlc }
 ImgLn     : ArrayLnImg;
 PCXLn     : ArrayLnPCX;
 RedLn,
 BlueLn,
 GreenLn,
 IntenLn   : Array80;
 Img       : pointer;

{--------------------------}
{ A fejlc felptse      }
{ 640x480 16 szn VGA kp }
{--------------------------}
procedure BuildHeader(BackgrColor : byte);
 const { a 16-szn alappaletta }
    PALETTEMAP: ArrayPal=
                {  R    G    B                    }
               (($00, $00, $00),  {  black        }
                ($00, $00, $AA),  {  blue         }
                ($00, $AA, $00),  {  green        }
                ($00, $AA, $AA),  {  cyan         }
                ($AA, $00, $00),  {  red          }
                ($AA, $00, $AA),  {  magenta      }
                ($AA, $55, $00),  {  brown        }
                ($AA, $AA, $AA),  {  lightgray    }
                ($55, $55, $55),  {  darkgray     }
                ($55, $55, $FF),  {  lightblue    }
                ($55, $FF, $55),  {  lightgreen   }
                ($55, $FF, $FF),  {  lightcyan    }
                ($FF, $55, $55),  {  lightred     }
                ($FF, $55, $FF),  {  lightmagenta }
                ($FF, $FF, $55),  {  yellow       }
                ($FF, $FF, $FF) );{  white        }
 var
   i : word;
 begin
   palettemap[0]:=palettemap[BackgrColor];
   with Header do
     begin
       Manufacturer:= 10;
       Version:= 5;
       Encoding:= 1;
       Bits_per_pixel := 1;
       Xmin := 0;
       Ymin := 0;
       Xmax := 639;
       Ymax := 479;
       Hdpi := 640;
       Vdpi := 480;
       ColorMap := PALETTEMAP;
       Reserved := 0;
       Nplanes  := 4; { Piros, Zld, Kk, Intenzits }
       B_l_p := 80;
       PaletteInfo := 1;
       HscreenSize := 0;
       VscreenSize := 0;
       for i := 74 to 127 do
           Filler [i] := 0;
     end;
 end;

{--------------------}
{ A PCX sor kdolsa }
{--------------------}
procedure EncPCXLine (PlaneLine : word);
 var
   This, Last, RunCount : byte;
   i, j : word;

 { kdolt fjlbars }
 procedure EncPut (Byt, Cnt :byte);
  const
    COMPRESS_NUM = $C0;  { a fels kt bit egy szmll }
  var
    Holder : byte;
  begin
    {$I-}
     if (Cnt = 1) and (COMPRESS_NUM <> (COMPRESS_NUM and Byt)) then
        blockwrite (PCXName, Byt,1) { egyszeres elforduls }
     else
        begin
          Holder := (COMPRESS_NUM or Cnt);
          blockwrite (PCXName, Holder, 1); { a sznismtlds szma }
          blockwrite (PCXName, Byt, 1);
        end;
    {$I+}
  end;

 begin
   i := 1;  { a PCXLn tmb indexe }
   RunCount := 1;
   Last := PCXLn [PlaneLine][i];
    for j := 1 to Header.B_l_p-1 do
      begin
        inc (i);
        This := PCXLn [PlaneLine][i];
        if This = Last then
          begin
            inc (RunCount);
            { az ismtlds maximlis rtke 63 }
            if RunCount = 63 then
               begin
                 EncPut (Last, RunCount);
                 RunCount := 0;
               end;
          end
        else
          begin
            if RunCount >= 1 then EncPut (Last, RunCount);
            Last := This;
            RunCount := 1;
          end;
      end;
   if RunCount >= 1 then  { maradt mg valami? }
      Encput (Last, RunCount);
 end;

{---------------------------------------------------}
{ Az aktulis (VGA) kptartalom PCX-fjlban mentse }
{ Name nven, BackColor httrsznnel               }
{---------------------------------------------------}
procedure WritePCX (Name:Str80; BackColor : byte);
 const
   XMAX = 639;
   YMAX = 479;
 var
   i, j, Size : word;
 begin
   BuildHeader(BackColor);
   assign(PCXName,Name);
   {$I-}
   rewrite(PCXName,1);
   blockwrite(PCXName,Header,sizeof (Header));
   {$I+}
   setviewport (0,0,XMAX,YMAX, ClipOn);
   Size := imagesize (0,0,XMAX,0); { egy sor lementse }
   getmem (Img,Size);

   { a kp sorainak feldolgozsa }
   for i := 0 to YMAX do
     begin
       getimage (0, i, XMAX, i, Img^);  { 1 kpsor az Img pufferbe }
       move (Img^, ImgLn, Size {326});
       { a kpsor sznsszetevkre bontsa }
       move(ImgLn[INTENSTART], IntenLn[1], Header.B_l_p);
       move(ImgLn[BLUESTART], BlueLn[1], Header.B_l_p);
       move(ImgLn[GREENSTART], GreenLn[1], Header.B_l_p);
       move(ImgLn[REDSTART], RedLn[1], Header.B_l_p);
       { a sznsszetevk PCX-kdolsa}
       PCXLn [1] := RedLn;
       PCXLn [2] := GreenLn;
       PCXLn [3] := BlueLn;
       PCXLn [4] := IntenLn; { 320 bjt/sor nincs tmrts }
       for j := 1 to Header.NPlanes do EncPCXLine (j);
     end;
   freemem (Img,Size);
   {$I-}
   close (PCXName);
   {$I+}
 end;

end.
