(* Mand87_1.pas *)

{----------------------------------------------------------
  Egyszer Mandelbrot halmaz rajzolsa
  A Mandpas program javtsa az iterci gpikdra trtn
  trsval.  A rajzols ideje mag=1 mellett:

  100 MHz IBM AT (Pentium) s VGA :  5.3 mp
 ----------------------------------------------------------}

{  $N+,E- 80x87    }

program Mandel;
{$N+,E-,R-,S-}
{$D-,L-}

uses crt,graph;

var
  graphdriver, graphmode:integer;

  restep, imstep : extended;  { a vals s a kpzetes rsz lpshossza }
  repart, impart : extended;              { a vals s a kpzetes rsz }
  x,y            : integer;             { aktulis kpernykoordintk }
  xasp, yasp     : word;                                { aspect ratio }
  maxx, maxy, maxc, cstep : integer;              { grafikai jellemzk }
  color : integer;

const
   restart = -2.267;  reend=1.0;         { a vals rsz rtkhatrai   }
   imstart = -1.125;                     { a kpzetes rsz kezdrtke }
   max : integer = 160;                  { maximlis itercis mlysg }
   mag = 1;                              { magnifikcis tnyez       }

                                    { fggvny az iterci elvgzsre }


function  iterate(re,im : extended): integer;
const cnst4:extended=4.0;

{A 80x87 regiszter stack-jnek hasznlata
           R7     -     re
           R6     -     im
           R5     -     4.0
           R4     -     x
           R3     -     y2
           R2     -     x2
           R1     -     y
           R0     -     x2+y2

}
{A 80x86 processzor stack-jnak hasznlata a rutinban

           re               10 byte  <- [BP+14]
           im               10 byte  <- [BP+ 4]
           near ret. cm     2 byte
           bp                2 byte  <- BP
           @result           2 byte  <- [BP-2]
           ST helye          2 byte  <- [BP-4]
}


begin
asm
         fld tbyte ptr re           { re     -> (R7)      }
         fld tbyte ptr im           { im     -> (R6)      }

         fld cnst4;                 { 4.0    -> (R5)      }
         xor bx,bx                  { k=0                 }
         fldz                       { x2 (R4) =0;         }
         fldz                       { y2 (R3) =0;         }
         fldz                       { x  (R2) =0;         }
         fldz                       { y  (R1) =0;         }

                                   { y:=2*x*y+im          }
        fmul st,st(3)              { y:=y*x               }
@iter:  fadd st,st(0)              { *2                   }
        fadd st,st(5)              { +im                  }

                                   { x:=x2-y2+re          }
        fincstp                    { sp -> R2-re ll      }
        fsub st,st(1)              { x2-y2                }
        fadd st,st(5)              { +re                  }
        fst  st(2)                 { x   trolsa  (R4)   }


                                   { x2 :=sqr(x)           }
        fmul st,st(0)              { x2 = sqrt(x)          }

        inc  bx                    { inc(k)                }
        fst  st(6)                 { x2 (R0) ideigl. tr.  }

                                   { y2 := sqr(y)          }
        fdecstp                    { sp R1-re ll          }
        fst  st(2)                 { y2=y                  }
        fmul st(2),st              { y2 (R3) = sqr(y)      }

                          {  until (x2+y2 > 4) or (k>=max) }
        fdecstp                    { sp -> x2 (r0) ideigl. }
        fadd st,st(3)              { x2+y2                 }
        fcomp st(5)                { if x2+y2>4.0 (R5)     }
        push ax                    { FSTSW helye           }
        fstsw word ptr [bp-4]      { ST trolsa           }
        fmul st,st(3)              { y (R3) = y(R1)* x(R4) }
        pop ax                     { ST -> AX              }
        sahf                       { Al -> Flags           }
        ja @ok                     { >4.0 -> vge          }
        cmp bx,max                 { k=max ?               }
        jb @iter                   { nem, iterl           }
@ok:    finit                      { 80x87 regiszterek <-0 }
        mov @result,bx             { iterate=k (eredmny)  }
end;
end;

                            { fggvny a kr/ciklois teszt elvgzsre }
function cycltest(zsx,re,im:extended):boolean;
var zr,zs,imq:extended;
begin
  imq := sqr(im);
  zr:=sqr(re) + imq;
  zs:=sqrt(zr-0.5*re+0.0625);
  if (16.0*zr*zs > 5*zs-4*re+1) and
      (sqr(re+1)+imq>0.0625)  then cycltest:=true
                                   else cycltest:=false;
end;

begin
  graphdriver:=detect;
  initgraph(graphdriver, graphmode,'');
  getaspectratio(xasp, yasp);
  maxx:=getmaxx;
  maxy:=getmaxy;
  maxc:=maxcolors+1; cstep:=max div (maxc);
  restep:=(reend-restart) /maxx;
  imstep:=restep * yasp / xasp;

  y:=0;
  impart:=imstart;
  while y<=maxy do
  begin
    repart := restart;
    x:=0;
    while x<=maxx do
    begin
      if cycltest(0.0625,repart,impart)
         then color:=iterate(repart,impart)
         else color:=max;
      putpixel(x,y,maxc*cstep-color);
      inc(x,mag);
      repart:=repart+restep*mag;
    end; { while x }
    if keypressed then    { billentynyomssal brmikor ki lehet lpni }
       begin
         readln;
         closegraph;
         exit;
       end;
    inc(y,mag);
    impart:=impart+imstep*mag;

  end;  {while y}
  readln;
  closegraph;
end.

