(* OOPDIR.PAS *)

{-------------------------------------------------------------
  A knyvtrstruktra  feldolgozshoz objektumokat definil
   modul. Az OOPFF.PAS hasznlja.
 -------------------------------------------------------------}

unit OOPDir;

{------------------------------------------------------------------------}
interface
uses Dos, OOPList;

type
  AttrType = Byte;
  FileStr = String[12];

  { Alap fjlbejegyzs-tpus }
  BaseEntryPtr = ^BaseEntry;
  BaseEntry = object(Node)
    Attr: AttrType;
    Time: Longint;
    Size: Longint;
    Name: FileStr;
    constructor Init;
    destructor Done; virtual;
    procedure ConvertRec(S:SearchRec);
    function FileName: FileStr; virtual;
    function FileExt: ExtStr; virtual;
    function FullName: PathStr; virtual;
    function FileTime: Longint; virtual;
    function FileAttr: AttrType; virtual;
    function FileSize: Longint; virtual;
    function IsDirectory: Boolean;
    end;

  { Fjlbejegyzs }
  FileEntryPtr = ^FileEntry;
  FileEntry = object(BaseEntry)
    constructor Init;
    destructor Done; virtual;
    procedure ForceExt(E:ExtStr);
    procedure ChangeName(P:PathStr); virtual;
    { Fjlnvcsere a memriban }
    procedure ChangePath(P:PathStr); virtual;
    procedure ChangeTime(T:Longint); virtual;
    procedure ChangeAttr(A:AttrType); virtual;
    procedure Erase; virtual;
    function Rename(NewName:PathStr): Boolean; virtual;
    { A fjl tnevezse a lemezen }
    function ResetTime: Boolean;
    function ResetAttr: Boolean;
    function SetTime(T:Longint): Boolean; virtual;
    function SetAttr(A:AttrType): Boolean; virtual;
    end;

  DirEntryPtr = ^DirEntry;
  DirEntry = object(FileEntry)
    DirEntries: List;
    constructor Init;
    constructor Clear;
    destructor Done; virtual;
    procedure FindFiles(FileSpec: FileStr; Attrib: AttrType);
    procedure FindDirectories(FileSpec: FileStr; Attrib: AttrType);
    end;


function ExtensionPos(FName : PathStr): Word;
function CurDir: PathStr;

{------------------------------------------------------------------------}
implementation

  {--------------------------------------------------------------------}
  { BaseEntry  metdusok                                               }
  {--------------------------------------------------------------------}
  constructor BaseEntry.Init;
    begin
    end;

  destructor BaseEntry.Done;
    begin
    end;

  procedure BaseEntry.ConvertRec;
    begin
    Name := S.Name;
    Size := S.Size;
    Time := S.Time;
    Attr := S.Attr;
    end;

  function BaseEntry.FileName;
    begin
    FileName := Name;
    end;

  function BaseEntry.FullName;
    begin
    FullName := Name;
    end;

  function BaseEntry.FileExt;
    var
      ep: word;
    begin
    ep := ExtensionPos(Name);
    if ep > 0 then
      FileExt := Copy(Name, Succ(ep), 3)
    else
      FileExt[0] := #0;
  end;


  function BaseEntry.FileAttr;
    begin
    FileAttr := Attr;
    end;

  function BaseEntry.FileSize;
    begin
    FileSize := Size;
    end;

  function BaseEntry.FileTime;
    begin
    FileTime := Time;
    end;

  function BaseEntry.IsDirectory;
    begin
    IsDirectory := (FileAttr and Dos.Directory) = Dos.Directory;
    end;

  {--------------------------------------------------------------------}
  { FileEntry metdusok                                                }
  {--------------------------------------------------------------------}
  constructor FileEntry.Init;
    begin
    BaseEntry.Init; { Az init metdusnak hvsa }
    Name := '';
    Size := 0;
    Time := $210000; {  1980. jan. 1., 12:00a }
    Attr := $00;   { ReadOnly  = $01;  - fjlattribtumok
                     Hidden    = $02;
                     SysFile   = $04;
                     VolumeID  = $08;
                     Directory = $10;
                     Archive   = $20;
                     AnyFile   = $3F; }
    end;

  destructor FileEntry.Done;
    begin
    BaseEntry.Done;
    end;

  function FileEntry.Rename;
    var
      F: File;
    begin
    Assign(F,FullName);
    System.Rename(F,NewName);

    if IOResult = 0 then
      begin
      ChangePath(NewName);
      Rename := True;
      end
    else
      Rename := False;
    end;

  procedure FileEntry.ForceExt;
    var
      ep: Word;
      TempBool: Boolean;
    begin
    ep := ExtensionPos(FullName);
    if ep > 0 then
      TempBool := Rename(Concat(Copy(FullName, 1, ep),FileExt))
    else
      TempBool := Rename(Concat(FullName,'.',FileExt));
    end;

  procedure FileEntry.ChangeName;
    begin
    Name := P;
    end;

  procedure FileEntry.ChangePath;
    begin
    Name := P;  { a FileEntry objektum nem kezel elrsi utat }
    end;

  procedure FileEntry.ChangeTime;
    begin
    Time := T;
    end;

  procedure FileEntry.ChangeAttr;
    begin
    Attr := A;
    end;

  procedure FileEntry.Erase;
    var
      F:File;
    begin
    Assign(F,FullName);
    Reset(F);
    System.Erase(F);
    Close(F);
    end;

  function FileEntry.ResetTime;
    var
      F:File;
    begin
    Assign(F,FullName);
    Reset(F);
    SetFTime(F,FileTime);
    ResetTime := IOResult = 0;
    Close(F);
    end;

  function FileEntry.SetTime;
    var
      F:File;
    begin
    Assign(F,FullName);
    Reset(F);
    SetFTime(F,T);
    SetTime := IOResult = 0;
    Close(F);
    end;

  function FileEntry.ResetAttr;
    var
      F:File;
    begin
    Assign(F,FullName);
    SetFAttr(F,FileAttr);
    ResetAttr := IOResult = 0;
    end;

  function FileEntry.SetAttr;
    var
      F:File;
    begin
    ChangeAttr(A);
    SetAttr := ResetAttr;
    end;

  {--------------------------------------------------------------------}
  { DirEntry  metdusok                                                }
  {--------------------------------------------------------------------}
  constructor DirEntry.Init;
    var
      TempNode: Node;
    begin
    FileEntry.Init;
    DirEntries.Delete;
    end;

  destructor DirEntry.Done;
    begin
    DirEntries.Delete;
    FileEntry.Done;
    end;

  constructor DirEntry.Clear;
    begin
    DirEntries.Clear;
    Init;
    end;

  procedure DirEntry.FindFiles;
    var
      DirInfo: SearchRec;
      TempFile: FileEntryPtr;
    begin
    FindFirst(FileSpec,Attrib,DirInfo);
    while (DosError = 0) do
      begin
      TempFile := New(FileEntryPtr,Init);
      TempFile^.ConvertRec(DirInfo);
      DirEntries.Append(TempFile);
      FindNext(DirInfo);
      end;
    end;

  procedure DirEntry.FindDirectories;
    var
      DirInfo: SearchRec;
      TempDir: DirEntryPtr;
    begin

    if FileSpec <> '' then
      FindFiles(FileSpec,Attrib and not Dos.Directory);

    FindFirst('*.*',Dos.Directory,DirInfo);
    while (DosError = 0) do
      begin
      if (DirInfo.Name[1] <> '.') and
         ((DirInfo.Attr and Dos.Directory) = Dos.Directory) then
         { Ha az els karakter '.', akkor a nv vagy '.' vagy '..' }
        begin
        TempDir := New(DirEntryPtr,Clear);
        TempDir^.ConvertRec(DirInfo);
        DirEntries.Append(TempDir);
        end;
      FindNext(DirInfo);
      end;

    TempDir := DirEntryPtr(DirEntries.First);
    while TempDir <> nil do
      begin
      if TempDir^.IsDirectory then
        begin
        ChDir(TempDir^.FileName);
        TempDir^.FindDirectories(FileSpec,Attrib);
        ChDir('..');
        end;
      TempDir := DirEntryPtr(DirEntries.Next(TempDir));
      end;
    end;

{--------------------------------------------------------------------}
{ Segdeljrsok                                                     }
{--------------------------------------------------------------------}

{ A fjl nevbl az elrsi t levlasztsa }
function ExtensionPos;
  var
    Index: Word;
  begin
  Index := Length(FName)+1;
  repeat
    dec(Index);
    until (FName[Index] = '.') OR (Index = 0);
  IF (Pos('\', Copy(FName, Succ(Index), SizeOf(FName))) <> 0) THEN Index := 0;
  ExtensionPos := Index;
  end;

{ Az aktulis knyvtr lekrdezse }
function CurDir;
  var
    P: PathStr;
  begin
  GetDir(0,P); { 0 = aktulis meghat }
  CurDir := P;
  end;

end.
