  {--------------------------------------------------------------}
  {			     SERIALU.INC                         }
  {								 }
  {	      Konstansdefnicik a SERIALU.PAS programhoz        }
  {--------------------------------------------------------------}

   PCOM1 = $3F8;  { a COM1 portcme }
   PCOM2 = $2F8;  { a COM2 portcme }

 { Az 8259-es programozhat megszaktsvezrl portcmei:        }
   IMR = $21;  { Interrupt Mask Register port }
   ICR = $20;  { Interrupt Control port       }
   { A hardveres megszakts vgt jelz rtk:      }
   EOI = $20;  { End Of Interrupt }
   { Maszkok }
   IRQ3 = $F7; { COM2 }
   IRQ4 = $EF; { COM1 }


 {--------------------------------------------------------------------
   A 8250 UART  10 regiszterrel rendelkezik, amelyeket  7 portcm
   felhasznlsval cmezhetnk meg. (A definilt cmek relatvek
   az alap portcmhez kpest.) A baud rate registerek (DLL s DLH)
   csak akkor aktvak, ha a Divisor-Latch Access-Bit (DLAB) rtke 1.
   (A DLAB az LCR 7. bitje.)
  --------------------------------------------------------------------}
   TXR = 0;      { Transmit register (WRITE) }
   RXR = 0;      { Receive register  (READ)  }
   IER = 1;      { Interrupt Enable          }
   IIR = 2;      { Interrupt ID              }
   LCR = 3;      { Line control              }
   MCR = 4;      { Modem control             }
   LSR = 5;      { Line Status               }
   MSR = 6;      { Modem Status              }
   DLL = 0;      { Divisor Latch Low         }
   DLH = 1;      { Divisor latch High        }

  {--------------------------------------------------------------
    A Line Control Register (LCR) tartalmazza a kommunikci
    paramtereit:
	  bit             rtelmezs
	  ---             ----------
	  0=1             00=5,  01=6,  10=7, 11=8  adatbit
	  2               stop bitek.
	  3               0=parity off, 1=parity on.
	  4               0=parity odd, 1=parity even.
	  5               A paritshiba fellbirlsa
	  6               BREAK.
	  7               DLAB
   --------------------------------------------------------------}
    NO_PARITY   = $00;
    EVEN_PARITY = $18;
    ODD_PARITY  = $08;

  {--------------------------------------------------------------
    A Line Status Register (LSR) bitjeinek jelentse:
	  bit             rtelmezs
	  ---             ----------
	  0               rvnyes adat van
	  1               Rfuts trtnt
	  2               Paritshiba
	  3               Keretezsei hiba
	  4               A Break llapot rzkelse
	  5               Az ad adatregisztere res
	  6               Az ad lptetregisztere res
	  7               Time out - off line.
   --------------------------------------------------------------}
    RCVRDY  = $01;
    OVRERR  = $02;
    PRTYERR = $04;
    FRMERR  = $08;
    BRKERR  = $10;
    XMTRDY  = $20;
    XMTRSR  = $40;
    TIMEOUT = $80;

  {--------------------------------------------------------------
    A Modem Output Control Register (MCR) bitjei:
	  bit             rtelmezs
	  ---             ----------
	  0               Data Terminal Ready = Ksz fogadni
	  1               Request To Send = Kldeni kvn
	  2               Auxillary output #1.
	  3               Auxillary output #2. (1=be kell lltani az
			  interrupt krshez)
	  4               Az UART kimeneteinek visszaktse
	  5=7             Nem hasznlt
   --------------------------------------------------------------}
    DTR    = $01;
    RTS    = $02;
    MC_INT = $08;

  {--------------------------------------------------------------
    A Modem Input Status Register (MSR) bitjei:
	  bit             rtelmezs
	  ---             ----------
	  0               A Clear To Send bemenet megvltozott
			  az utols olvass ta
	  1               A Data Set Ready bemenet megvltozott
			  az utols olvass ta
	  2               Az Ring Indicator bemenet aktv llapotba
			  kerlt az utols olvass ta
	  3               A Data Carrier Detect bemenet megvltozott
			  az utols kiolvassa ta
	  4               Clear To Send  = adsra ksz
	  5               Data Set Ready = adat ksz
	  6               Ring Indicator = csengets jelz
	  7               Data Carrier Detect = az adatviv rzkelhet
   --------------------------------------------------------------}
    CTS = $10;
    DSR = $20;

  {--------------------------------------------------------------
    Az Interrupt Enable Register (IER) hasznlt bitjei:
	  bit             rtelmezs
	  ---             ----------
	  0               Megszaktskrs, ha adat jtt
	  1               Megszaktskrs, ha az ad adatregisztere res
	  2               Megszaktskrs, ha adatfogadsi hiba lp fel
	  3               Megszaktskrs, ha modem llapot megvltozik
	  4=7             Nem hasznlt.
   --------------------------------------------------------------}
    RX_INT    =     $01;

  {--------------------------------------------------------------
    Az Interrupt Identification Register (IIR) bitjei:
	  bit             rtelmezs
	  ---             ----------
	  0         Megszaktskrs jelzse
	  1=2       Megszakts azonost:
			  00=A modem llapot megvltozott
			  01=Az ad adatregisztere res
			  10=Adat rkezett
			  11=Adatfogadsi hiba lpett fel
	  3=7             Nem hasznlt
   --------------------------------------------------------------}
    RX_ID   = $04;
    RX_MASK = $07;




















