(* SERIALU.PAS *)

unit Serialu;

{----------------------------------------------------------------}
interface
{----------------------------------------------------------------}
  type      SerialPort = (COM1, COM2);
  type      ParityType = (Odd,Even,None);

  { Karakterek szma az inputpufferben }
  function  AskCharNum (pt: SerialPort): integer;
  { Az inputpuffer trlse }
  procedure ClearBuf (pt: SerialPort);
  { Karakter olvassa a soros portrl }
  function  GetChar (pt: SerialPort): char;
  { Sztring olvassa a soros portrl }
  function  GetLine (pt: SerialPort): string;
  { A kommunikci inicializlsa }
  procedure InitCOM (pt: SerialPort);
  { Sztring rsa a soros portra }
  procedure PutLine (pt: SerialPort; OutStr: string);
  { Karakter rsa a soros portra }
  procedure PutChar (pt: SerialPort; c: char);
  { A kommunikci lezrsa }
  procedure ResetCOM (pt: SerialPort);
  { A kommunikcis protokoll belltsa }
  procedure SetProtocol (pt: SerialPort; Baud,Data,Stop: integer; Par: ParityType);
  { A tlfutsi id belltsa }
  procedure SetTimeOut (pt: SerialPort; time: word);


{----------------------------------------------------------------}
implementation
{----------------------------------------------------------------}
uses  crt, Dos;
type
  ComDescr = array[COM1..COM2] of byte;

const
  {$I SERIALU.INC konstansokat tartalmaz fjl }
  BufSize            = 3000; { az inputpuffer mrete }
  IMaskEn : ComDescr = (IRQ4, IRQ3); { az IRQ maszkok }
  IVec    : ComDescr = ($0C,$0B);    { a megszaktsvektor sorszma }

type
  PortRec = record
              Buffer     : array[1..BufSize] of byte;  { inputpuffer }
              InPointer  : integer; { belpsi mutat (szabad) }
              OutPointer : integer; { kilpsi mutat }
              Count      : integer; { a karakterek szma a pufferben }
              VecOld     : pointer; { a rgi megszaktsvektor}
              TimeOut    : boolean; { a GetChar hvsakor tlfuts trtnt }
              TOCntMax   : word;    { maximlis vrakozs a tlfutsig }
              ITMask     : byte;    { elmentett IMR rtk }
            end;

var
  PortVar   : array[COM1..COM2] of PortRec; { porttmb }
  padr      : word; { a port fizikai cme }
  cpt       : Serialport;
  actport   : array[COM1..COM2] of boolean; { aktivizlt port }
  exitvec   : pointer; { az eredeti kilpsi vektor }
  InByte    : integer;

procedure disable; assembler;
 asm cli end;

procedure enable; assembler;
 asm sti end;

{ ---------------- }
{ Kilpsi eljrs }
{ ---------------- }
procedure sexitproc; far;
 var cpt : serialport;
 begin
   for cpt:=COM1 to COM2 do
     if actport[cpt] then ResetCom(cpt);
   erroraddr := nil;
   exitproc  := exitvec; { Norml kilps }
 end;

{------------------------------}
{ A port cmnek meghatrozsa }
{------------------------------}
function CalcAdr (pt: SerialPort): word;
 begin
   case pt of
     COM1 : CalcAdr:= PCOM1;
     COM2 : CalcAdr:= PCOM2;
   end;
 end;

{------------------------------------}
{ Karakterek szma az inputpufferben }
{------------------------------------}
function  AskCharNum (pt: SerialPort): integer;
 begin
   AskCharNum := PortVar[pt].Count;
 end;

{-----------------------------------}
{ A tlfutsi id belltsa (msec) }
{-----------------------------------}
procedure SetTimeOut (pt: SerialPort; time: word);
 begin
   PortVar[pt].TOCntMax := time;
 end;

{-----------------------------------}
{ A COM1 megszaktskezel eljrsa }
{-----------------------------------}
procedure ISRCOM1; interrupt;
 begin
   if (port[PCOM1 + IIR] and RX_MASK) = RX_ID then
     begin
       InByte := port[PCOM1 + RXR]; { 1 bjt olvassa a portrl }
       with PortVar[COM1] do
       if Count < BufSize then
         begin
           Buffer[InPointer] := InByte; { a bjt trolsa a pufferben }
           inc(Count);
           inc(InPointer);
           if InPointer > BufSize then InPointer := 1;
         end;
     end;
   {End Of Interrupt 8259 parancs}
   port[ICR]:=EOI;
 end;

{-----------------------------------}
{ A COM2 megszaktskezel eljrsa }
{-----------------------------------}
procedure ISRCOM2; interrupt;
 begin
   if (port[PCOM2 + IIR] and RX_MASK) = RX_ID then
     begin
       InByte := port[PCOM2 + RXR]; { 1 bjt olvassa a portrl }
       with PortVar[COM2] do
       if Count < BufSize then
         begin
           Buffer[InPointer] := InByte;  { a bjt trolsa a pufferben }
           inc(Count);
           inc(InPointer);
           if InPointer > BufSize then InPointer := 1; { krkrs puffer }
         end;
     end;
   {End Of Interrupt 8259 parancs}
   port[ICR]:=EOI;
end;

{-------------------------------}
{ A kommunikci inicializlsa }
{-------------------------------}
procedure InitCOM (pt : SerialPort);
 var
   sv : byte;
 begin
   actport[pt]:=true; { az aktivizlt port megjellse }
   padr := CalcAdr (pt);
   with PortVar[pt] do
     begin
       InPointer := 1;  { a pufferkezelse inicializlsa }
       Count := 0;
       GetIntVec (IVec[pt],VecOld);
       case pt of
         COM1: SetIntVec (IVec[pt],addr(ISRCOM1)); { COM1 megszaktskezel }
         COM2: SetIntVec (IVec[pt],addr(ISRCOM2)); { COM2 megszaktskezel }
       end;
       disable;
       port[padr+MCR] := port[padr+MCR] or MC_INT;
       port[padr+LCR] := port[padr+LCR] and $7F; { a DLAB trlse }
       { a megszaktsok engedlyezse a 8250-en }
       port[padr+IER] := RX_INT;
       { megszaktsi maszk a 8259-en }
       ITMask:=port[IMR];
       port[IMR] := ITMask and IMaskEn[pt];
     end;
   port[padr + MCR]:= port[padr + MCR] or DTR or RTS;
   enable;
 end;

{-------------------------}
{ A kommunikci lezrsa }
{-------------------------}
procedure ResetCOM (pt : SerialPort);
 begin
   actport[pt]:=false;  { a lezrt port megjellse }
   disable;
   { a megszaktsi maszk visszalltsa (tilts) 8259-en: }
   port[IMR] := portVar[pt].ITMask; {port[IMR] or not(IMaskEn[pt]);}
   { a megszaktsok tiltsa a 8250-en: }
   padr := CalcAdr (pt);
   port[padr+MCR] := port[padr+MCR] and not(MC_INT);
   port[padr+IER] := $00;
   port[padr+MCR] := 0;
   { az eredeti megszaktsvektor visszalltsa: }
   enable;
   SetIntVec (IVec[pt], PortVar[pt].VecOld);
 end;

{--------------------------------------}
{ A kommunikcis protokoll belltsa:
       pt .... Port (COM1, COM2)
       Baud .. Baudrate (110,150,300,600,1200,2400,4800,9600)
       Data .. adatbitek szma (7,8)
       Stop .. stopbitek szma (1,2)
       Par ... paritsbit (None, Odd, Even) }
{--------------------------------------}
procedure SetProtocol (pt : SerialPort;
                       Baud, Data, Stop: integer; Par: ParityType);
 var
   BaudBit, DataBit, ParBit, StopBit : byte;
   regs : registers;
   sv   : byte;
 begin
   padr := CalcAdr (pt);
   sv := port[padr+IER]; { a megszaktsa llapot elmentse }
   case Baud of
      110 : BaudBit := 0;
      150 : BaudBit := 1;
      300 : BaudBit := 2;
      600 : BaudBit := 3;
     1200 : BaudBit := 4;
     2400 : BaudBit := 5;
     4800 : BaudBit := 6;
     9600 : BaudBit := 7
     else   BaudBit := 7
   end;
   case Data of
       7 : DataBit := 0;
       8 : DataBit := 1
      else DataBit := 1
   end;
   case Par of
     None : ParBit := 0;
     Odd  : Parbit := 1;
     Even : ParBit := 3;
     else   ParBit := 0
   end;
   case Stop of
       1 : StopBit := 0;
       2 : StopBit := 1
      else StopBit := 0
   end;
   { A paramterek belltsa a BIOS segtsgvel}
   with Regs do
     begin
       DX := ord(pt);
       AX := 32*BaudBit + 8*ParBit + 4*StopBit + 2 + DataBit;
       Intr($14,Regs);
     end;
   port[padr+IER] := sv; { a megszaktsa llapot visszalltsa }
 end;

{------------------------}
{ Az inputpuffer trlse }
{------------------------}
procedure ClearBuf (pt : SerialPort);
 begin
   port[IMR] := not (IMaskEn[pt]) or port[IMR]; { a megszakts tiltsa 8259-en }
   with PortVar[pt] do
     begin
       Count := 0;
       InPointer := 1;
       OutPointer := InPointer;
     end;
   port[IMR] := IMaskEn[pt] and port[IMR]; { a megszakts engedlyezse 8259-en }
 end;

{-----------------------------------}
{ Karakter olvassa a soros portrl }
{-----------------------------------}
function GetChar (pt : SerialPort): char;
 var
   TimeOutCnt  : integer;
 begin
   with PortVar[pt] do
     begin
       TimeOut := false;
       TimeOutCnt := TOCntMax;
       repeat { vrakozs }
         Delay (1);
         dec (TimeOutCnt);
       until (Count > 0) or (TimeOutCnt = 0);
       if TimeOutCnt = 0
         then begin
                GetChar := chr(0);
                timeOut := True;
              end
         else begin
                port[IMR] := not (IMaskEn[pt]) or port[IMR];
                GetChar := chr(Buffer[OutPOinter]);
                OutPointer := OutPointer + 1;
                Count := COunt - 1;
                if OutPointer > BufSize then OutPointer := 1;
                port[IMR] := IMaskEn[pt] and port[IMR];
              end;
     end;
 end;

{----------------------------------}
{ Sztring olvassa a soros portrl }
{----------------------------------}
function GetLine (pt : SerialPort): string;
 var
   c : char;
   svstr : string;
 begin
   svstr := '';
   repeat
     c := GetChar(pt);
     if c >= ' ' then svstr := svstr + c;
   until (c = chr(13)) or (length(svstr) = 255);
   GetLine := svstr;
 end;

{-------------------------------}
{ Karakter rsa a soros portra }
{-------------------------------}
procedure PutChar (pt : SerialPort; c : char);
 begin
   padr := CalcAdr (pt);
   port[padr + MCR]:= MC_INT or DTR or RTS;
   { vrakozs a Clear To Send jelre }
   repeat
   until (port[padr+MSR] and CTS) <> 0;
   { vrakozs a kszenlti jelre }
   repeat
   until (port[padr+LSR] and XMTRDY) <> 0;
   port [padr+TXR] := ord(c);
 end;


{------------------------------}
{ Sztring rsa a soros portra }
{------------------------------}
procedure PutLine (pt: SerialPort; OutStr: string);
 var
   i : integer;
 begin
   for i:= 1 to length(OutStr) do
     PutChar (pt,OutStr[i]);
 end;

{----------------------------------------------------------------}
{ Inicializcis rsz                                            }
{----------------------------------------------------------------}
begin
  exitvec:=exitproc;
  exitproc:=@sexitproc;

  for cpt := COM1 to COM2 do
   begin
     actport[cpt]:=false;
     with PortVar[cpt] do
       begin
         GetIntVec (IVec[cpt],VecOld);
         TOCntMax := 5000;
         InPointer := 1;
         OutPointer := 1;
         Count := 0;
       end;
   end;
 end.